@@ -22,10 +22,10 @@ def register():
         CommunicationAttachment,
         DocumentConfiguration,
         DocumentLendingDocument,
-        DocumentCommunicationEmployee,
+        DocumentCommunicationUser,
         CommunicationLetter,
         DocumentRecordInventoryStart,
-        SearchDocumentFileStart,
+        AddAttachmentStart,
         TRDStart,
         module='document', type_='model')
     Pool.register(
@@ -35,6 +35,6 @@ def register():
         module='document', type_='report')
     Pool.register(
         DocumentRecordInventory,
-        SearchDocumentFile,
+        AddAttachment,
         TRD,
         module='document', type_='wizard')
