@@ -1,24 +1,37 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from os import path as os_path
-import datetime
+try:
+    from cStringIO import StringIO
+except:
+    from StringIO import StringIO
+
+import os
+from datetime import datetime, date
+from io import BufferedReader, BytesIO
+from pyPdf import PdfFileWriter, PdfFileReader
+from reportlab.pdfgen import canvas
+from reportlab.lib.pagesizes import letter
+from reportlab.lib.colors import HexColor
+
 from trytond.pool import PoolMeta, Pool
 from trytond.model import ModelView, ModelSQL, Workflow, fields
-from trytond.pyson import Eval
+from trytond.pyson import Eval, And, In, Not
 from trytond.modules.company import CompanyReport
 from trytond.wizard import Wizard, StateTransition, StateView, Button
+from trytond.transaction import Transaction
+from trytond.config import config
+
 
 __all__ = ['Communication', 'CommunicationAttachment',
-    'DocumentCommunicationEmployee', 'CommunicationLetter',
-    'CommunicationLetterReport', 'SearchDocumentFileStart',
-    'SearchDocumentFile']
+    'DocumentCommunicationUser', 'CommunicationLetter',
+    'CommunicationLetterReport', 'AddAttachmentStart',
+    'AddAttachment']
 __metaclass__ = PoolMeta
 
 _STATES = {
     'readonly': Eval('state') != 'draft',
 }
-
-
+EXTENSION = '.pdf'
 
 class Communication(Workflow, ModelSQL, ModelView):
     'Communication'
@@ -28,8 +41,8 @@ class Communication(Workflow, ModelSQL, ModelView):
     description = fields.Char('Description', required=True, states=_STATES)
     party = fields.Many2One('party.party', 'Party', required=False,
         states=_STATES)
-    user_target = fields.Many2Many('company.employee-document.communication',
-        'document_communication', 'employee', 'Employee Target',
+    users_target = fields.Many2Many('res.user-document.communication',
+        'document_communication', 'user', 'User Target',
         states=_STATES)
     date_communication = fields.DateTime('Date',
             readonly=True)
@@ -42,8 +55,7 @@ class Communication(Workflow, ModelSQL, ModelView):
         ('incoming', 'Incoming'),
         ('outgoing', 'Outgoing'),
         ('internal', 'Internal'),
-        (None, ''),
-        ], 'Kind', select=True, states=_STATES)
+        ], 'Kind', select=True, readonly=True, states=_STATES)
     state = fields.Selection([
         ('draft', 'Draft'),
         ('signed', 'Signed'),
@@ -56,7 +68,13 @@ class Communication(Workflow, ModelSQL, ModelView):
         ('high', 'High'),
         ], 'Priority', required=True, select=True, states=_STATES)
     attachments = fields.One2Many('document.communication.attachment', 'communication', 
-            'Attachments', states=_STATES)
+            'Attachments', states={
+                'readonly': Eval('state') != 'draft',
+                'invisible': And(
+                    Not(In(Eval('_user', '0'), Eval('users_target', []))),
+                    Eval('create_uid') != Eval('_user', '0'),
+                    )
+            })
 
     @classmethod
     def __setup__(cls):
@@ -80,11 +98,14 @@ class Communication(Workflow, ModelSQL, ModelView):
                 'signed': {
                     'invisible': Eval('state') != 'draft',
                     },
+                'add_attachment': {
+                    'invisible': Eval('state') != 'draft',
+                    },
                 })
 
     @staticmethod
     def default_date_communication():
-        return datetime.datetime.now()
+        return datetime.now()
 
     @staticmethod
     def default_state():
@@ -116,7 +137,13 @@ class Communication(Workflow, ModelSQL, ModelView):
     @Workflow.transition('checked')
     def checked(cls, docs):
         for doc in docs:
-            doc.write([doc], {'date_checked': datetime.date.today()})
+            doc.write([doc], {'date_checked': date.today()})
+
+    @classmethod
+    @ModelView.button_action('document.act_add_attachment')
+    def add_attachment(cls, docs):
+        for doc in docs:
+            doc.set_number()
 
     def set_number(self):
         if self.reference:
@@ -130,26 +157,48 @@ class Communication(Workflow, ModelSQL, ModelView):
         seq = configuration.document_communication_sequence.id
         self.write([self], {'reference': Sequence.get_id(seq)})
 
+    def add_new_attachment(self, vals):
+        Attachment = Pool().get('document.communication.attachment')
+        attachment, = Attachment.create([{
+            'name': vals['name'],
+            'communication': self.id,
+            'date': vals['date'],
+            'path': vals['path'],
+            'description': vals['description'],
+            'kind': vals['kind'],
+            'document_url': vals['document_url'],
+        }])
+        return attachment
+
 
 class CommunicationAttachment(ModelSQL, ModelView):
     'Communication Attachment'
     __name__ = 'document.communication.attachment'
     _rec_name = 'description'
-    name = fields.Char('Name', required=False)
+    name = fields.Char('Name', required=True, states={
+            'readonly': True,
+            })
     communication = fields.Many2One('document.communication', 'Communication',
-            required=False)
-    document_url = fields.Char('Document Url', required=True)
-    date = fields.Date('Date', required=False)
+            required=True)
+    document_url = fields.Char('Document Url', states={
+            'required': True,
+            'readonly': True,
+            })
+    date = fields.Date('Date', required=True)
     employee = fields.Many2One('company.employee', 'Employee',
             required=False)
+    path = fields.Char('Path', states={
+            'required': False,
+            'readonly': True,
+            })
     description = fields.Text('Description', required=False)
     is_url_valid = fields.Function(fields.Boolean('Is Url Valid'),
-        'get_is_url_valid')
+            'get_is_url_valid')
     kind = fields.Selection([
             ('incoming', 'Incoming'),
             ('outgoing', 'Outgoing'),
             ('internal', 'Internal'),
-            ], 'Kind', select=True)
+            ], 'Kind', select=True, readonly=True)
     responses = fields.One2Many('document.communication.attachment',
             'parent', 'Responses')
     parent = fields.Many2One('document.communication.attachment', 'Parent',
@@ -158,13 +207,9 @@ class CommunicationAttachment(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(CommunicationAttachment, cls).__setup__()
-        cls._buttons.update({
-                'search_file': {
-                    'invisible': False,
-                    },})
 
     def get_is_url_valid(self, name):
-        if self.document_url and os_path.isfile(self.document_url):
+        if self.document_url and os.path.isfile(self.document_url):
             return True
         return False
 
@@ -174,46 +219,162 @@ class CommunicationAttachment(ModelSQL, ModelView):
 
     @staticmethod
     def default_date():
-        return datetime.date.today()
-
-    @classmethod
-    @ModelView.button_action('document.act_search_file')
-    def search_file(cls, docs):
-        print "se ejecuta search"
-        pass
+        return date.today()
 
 
-class SearchDocumentFileStart(ModelView):
-    'Search Document File'
-    __name__ = 'document.communication.attachment_search_file.start'
-    filepath = fields.Binary('Filepath', required=True)
+class AddAttachmentStart(ModelView):
+    'Add Attachment Start'
+    __name__ = 'document.communication.add_attachment.start'
+    name = fields.Char('Name Document', required=True)
+    buff_file = fields.Binary('Filepath', filename='name_document', required=True)
+    date = fields.Date('Date', required=False)
+    description = fields.Text('Description', required=False)
+    kind = fields.Selection([
+            ('incoming', 'Incoming'),
+            ('outgoing', 'Outgoing'),
+            ('internal', 'Internal'),
+            ], 'Kind', select=True)
+
+    @staticmethod
+    def default_date():
+        return date.today()
+
+    @staticmethod
+    def default_kind():
+        return 'incoming'
 
 
-class SearchDocumentFile(Wizard):
-    'Search Document File'
-    __name__ = 'document.communication.attachment_search_file'
+class AddAttachment(Wizard):
+    'Add Attachment'
+    __name__ = 'document.communication.add_attachment'
 
-    start = StateView('document.communication.attachment_search_file.start',
-        'document.communication_attachment_search_file_view_form', [
+    start = StateView('document.communication.add_attachment.start',
+        'document.communication_add_attachment_view_form', [
                 Button('Cancel', 'end', 'tryton-cancel'),
                 Button('Add', 'search_', 'tryton-ok', default=True),
             ])
     search_ = StateTransition()
 
+    @classmethod
+    def __setup__(cls):
+        super(AddAttachment, cls).__setup__()
+        cls._error_messages.update({
+                'document_pdf_invalid': 'Document PDF invalid!',
+                'missing_path_fs': 'Missing path to Filesystem!',
+                'filename_duplicated': 'File name duplicated!',
+                })
+
     def transition_search_(self):
-        #print self.start.filepath
-        print type(self.start.filepath)
+        dest_path = config.get('path_sgd', 'data_path')
+        if not dest_path or not os.path.exists(dest_path):
+            self.raise_user_error('missing_path_fs')
+        year = str(date.today().year)
+        full_path = os.path.join(dest_path, year)
+        if not os.path.exists(full_path):
+            os.mkdir(full_path, 0755)
+        pool = Pool()
+        Communication = pool.get('document.communication')
+        Company = pool.get('company.company')
+        communication = Communication(Transaction().context.get('active_id'))
+        company = Company(Transaction().context.get('company'))
+
+        name_doc = communication.reference + '_' + self.start.name + EXTENSION
+        new_doc = os.path.join(full_path, name_doc)
+        if os.path.exists(new_doc):
+            self.raise_user_error('filename_duplicated')
+        self.set_document(
+            new_doc, 
+            self.start.buff_file,
+            company.party.name,
+            communication.reference,
+            communication.create_uid.name,
+            communication.create_date,
+        )
+        if not self.start.description:
+            description = name_doc
+        else:
+            description = self.start.description
+
+        data = {
+            'name': name_doc,
+            'date': self.start.date,
+            'path': new_doc,
+            'description': description,
+            'kind': self.start.kind,
+            'document_url': new_doc,
+        }
+        communication.add_new_attachment(data)
         return 'end'
 
+    def set_document(self, name_doc, buff_file, company, reference, 
+            create_uid, create_date):
+        if not reference:
+            reference = 'RAD-00000001'
+        ref = u'Radicado No. ' + reference
+        date_ = 'Fecha: ' + str(create_date)[:19]
+        user = 'Recibio: ' + create_uid
 
-class DocumentCommunicationEmployee(ModelSQL):
-    'Document Communication Employee'
-    __name__ = 'company.employee-document.communication'
-    _table = 'company_employee_document_communication'
+        # read your existing PDF
+        bytes_io = BytesIO(buff_file)
+        reader = BufferedReader(bytes_io)
+        try:
+            existing_pdf = PdfFileReader(reader)
+        except:
+            self.raise_user_error('document_pdf_invalid')
+        page_zero = existing_pdf.getPage(0)
+        x_dim = int(page_zero.mediaBox.getUpperRight_x())
+        y_dim = int(page_zero.mediaBox.getUpperRight_y())
+
+        # create a new PDF with Reportlab
+        # Frame
+        string_io = StringIO()
+        can = canvas.Canvas(string_io, pagesize=letter)
+        can.setLineWidth(0.5)
+        can.setFillColor(HexColor(0xFCFCFF), alpha=0.65)
+        can.setDash([3, 2, 1, 2], phase=0)
+        can.setStrokeColor(HexColor(0xB3B7BA))
+        can.rect((x_dim - 145), (y_dim - 52), 135, 45, fill=1)
+
+        # Text Frame
+        can.setFont("Helvetica-Oblique", 8)
+        can.setFillColorRGB(0.2, 0.2, 0.2)
+        can.drawString((x_dim - 140), (y_dim - 18), company)
+        can.setFont("Helvetica-Bold", 8)
+        can.drawString((x_dim - 140), (y_dim - 28), ref)
+        can.setFont("Helvetica", 8)
+        can.drawString((x_dim - 140), (y_dim - 38), date_)
+        can.drawString((x_dim - 140), (y_dim - 48), user)
+        can.save()
+
+        #move to the beginning of the StringIO buffer
+        string_io.seek(0)
+        new_pdf = PdfFileReader(string_io)
+
+        output = PdfFileWriter()
+        # add the "watermark" (which is the new pdf) on the existing page
+        for i in range(existing_pdf.getNumPages()):
+            page = existing_pdf.getPage(i)
+            if i == 0:
+                try:
+                    page.mergePage(new_pdf.getPage(0))
+                except:
+                    self.raise_user_error('document_pdf_invalid')
+            output.addPage(page)
+
+        # finally, write "output" to a real file
+        outputStream = open(name_doc, "wb")
+        output.write(outputStream)
+        outputStream.close()
+
+
+class DocumentCommunicationUser(ModelSQL):
+    'Document Communication User'
+    __name__ = 'res.user-document.communication'
+    _table = 'res_user_document_communication'
     document_communication = fields.Many2One('document.communication',
         'Document Communication', ondelete='CASCADE', select=True, 
         required=True)
-    employee = fields.Many2One('company.employee', 'Employee', select=True, 
+    user = fields.Many2One('res.user', 'User', select=True, 
         required=True, ondelete='RESTRICT')
 
 
@@ -258,26 +419,26 @@ class CommunicationLetter(Workflow, ModelSQL, ModelView):
 
     @staticmethod
     def default_date():
-        return datetime.date.today()
+        return date.today()
 
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
-    def draft(cls, letters):
+    def draft(cls, docs):
         pass
 
     @classmethod
     @ModelView.button
     @Workflow.transition('check')
-    def check(cls, letters):
+    def check(cls, docs):
         pass
 
     @classmethod
     @ModelView.button
     @Workflow.transition('done')
-    def done(cls, letters):
-        for letter in letters:
-            letter.set_sequence()
+    def done(cls, docs):
+        for doc in docs:
+            doc.set_sequence()
 
     def set_sequence(self):
         return
