@@ -41,10 +41,10 @@ class DocumentRecord(Workflow, ModelSQL, ModelView):
     location = fields.Many2One('document.location', 'Location',
             states=_STATES)
     final = fields.Selection([
-            ('conservacion', 'Conservacion'),
-            ('eliminacion', 'Eliminacion'),
-            ('microfilmacion', 'Microfilmacion'),
-            ('seleccion', 'Seleccion'),
+            ('conservation', 'Conservation'),
+            ('elimination', 'Elimination'),
+            ('microfilm', 'Microfilm'),
+            ('selection', 'Selection'),
             ], 'Final', required=True)
     state = fields.Selection([
             ('draft', 'Draft'),
